package PSkan::DB;

#
#
# PSkan::DB
# 
# Tie::Hash interface to a db for storing 
# data retrieved for wireless networks using 
# a little perl scanner
#
# Eric Maland (poon at muppetlabs.com) 09/2001
#
#

our $VERSION = '0.01';

use strict;
use Fcntl;
use DB_File;

my $db_path = $ENV{HOME} . "/.perlskan.db";

sub open_db_with_mode {
    my $mode = shift;
    my $db_handle = {};

    {
	my %foo = ();
	my $db = tie(%foo, 'DB_File', $db_path, $mode, 0666, $DB_BTREE) 
	    || die "Failed to open $db_path: $!";

	$db_handle->{dbpath} = $db_path;
	$db_handle->{dbref}  = $db;
	$db_handle->{dbhash} = \%foo;
    }

    return $db_handle;
}

sub TIEHASH {
    my $class = shift;
    my $self = {};

    my $db = open_db_with_mode(O_RDWR|O_CREAT);

    $self->{db} = $db;
    
    bless($self, $class);
}

sub FIRSTKEY {
    my $self = shift;
    
    return (each %{$self->{db}->{dbhash}})
}

sub NEXTKEY {
    my ($self, $last) = @_;

    return (each %{$self->{db}->{dbhash}})
}

sub make_record {
    my $data = shift;
    my $rec  = join("\001", $data->{link}, $data->{freq}, $data->{bitrate},
		    $data->{latdir}, $data->{lat}, $data->{longdir}, $data->{long});
    return $rec;
}

sub STORE {
    my ($self, $key, $data) = @_;    

    return $self->{db}->{dbhash}{$key} = make_record($data);
}

sub parse_record {
    my $stuff = shift;
    my @data = split(/\001/, $stuff);
    my $rec = {};

    $rec->{link} = shift @data;
    $rec->{freq} = shift @data;
    $rec->{bitrate} = shift @data;

    $rec->{latdir} = shift @data;
    $rec->{lat}    = shift @data;
    $rec->{longdir}    = shift @data;
    $rec->{long}    = shift @data;

    return $rec;
}

sub FETCH {
    my ($self, $key) = @_;

    return parse_record($self->{db}->{dbhash}{$key});
}

sub DELETE {
    my ($self, $key) = @_;
    
    return delete $self->{db}->{dbhash}{$key};
}

sub EXISTS {
    my ($self, $key) = @_;

    return exists $self->{db}->{dbhash}{$key};
}

sub CLEAR {
    my $self = shift;
    undef(%{$self->{db}->{dbhash}});
}

sub DESTROY {
    my ($self) = @_;
    undef $self;
}

1;

__END__

=head1

This is your standard Tie::Hash module.  

tie(%x, PSkan::DB)

If you don't know how to operate on a hash, you've got issues.  I'll
document the format of the data in the database here soon enough.
