#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include <linux/wireless.h>
#include <sys/ioctl.h>
#include <errno.h>

MODULE = PSkan::Wireless		PACKAGE = PSkan::Wireless

char *
wireless_get_essid(sock,ifname)
	int sock;
	char *ifname;
	PREINIT:
	  struct iwreq wrq;
	  char essid_buf[IW_ESSID_MAX_SIZE + 1];
	CODE:
          memset(&wrq, 0, sizeof(wrq));
          memset(essid_buf, 0, sizeof(essid_buf));

	  wrq.u.essid.pointer = (caddr_t)essid_buf;          
          wrq.u.essid.flags   = 1;

          strcpy(wrq.ifr_name, ifname);

          if(ioctl(sock, SIOCGIWESSID, &wrq) >= 0) {
            RETVAL = (char*)wrq.u.essid.pointer;
	  }
	OUTPUT:
	  RETVAL

int
wireless_set_essid(sock,ifname,essid)
	int sock;
	char *ifname;
	char *essid;
	PREINIT:
	  struct iwreq wrq;
	  char essid_buf[IW_ESSID_MAX_SIZE + 1];
	CODE:
          memset(&wrq, 0, sizeof(wrq));
	  memset(essid_buf, 0, sizeof(essid_buf));

	  if(essid == NULL || essid[0] == '\0') {
            wrq.u.essid.flags   = 0;
          } else {
	    strcpy(essid_buf, essid);
            wrq.u.essid.flags   = 1;
          }
          
          strcpy(wrq.ifr_name, ifname);

	  wrq.u.essid.pointer = (caddr_t)essid_buf;
	  wrq.u.essid.length  = strlen(essid_buf) + 1;

          if(ioctl(sock, SIOCSIWESSID, &wrq) >= 0) {
            RETVAL = 1;
	  } else {
 	    warn("ioctl(): %s\n", strerror(errno));
	    RETVAL = 0;
          }
	OUTPUT:
	  RETVAL

char *
wireless_get_ap_mac(sock,ifname)
	int sock;
	char *ifname;
	PREINIT:
	  struct iwreq wrq;
	CODE:
          memset(&wrq, 0, sizeof(wrq));

          strcpy(wrq.ifr_name, ifname);

          if(ioctl(sock, SIOCGIWAP, &wrq) >= 0) {
	    char buf[64];
	    char *ptr = wrq.u.ap_addr.sa_data;

	    sprintf(buf, "%02X:%02X:%02X:%02X:%02X:%02X",
        (ptr[0] & 0xFF), (ptr[1] & 0xFF), (ptr[2] & 0xFF),
        (ptr[3] & 0xFF), (ptr[4] & 0xFF), (ptr[5] & 0xFF));

            RETVAL = buf;
	  } else {
	    XSRETURN_UNDEF;
          }
	OUTPUT:
	  RETVAL

long
wireless_get_bit_rate(sock,ifname)
	int sock;
	char *ifname;
	PREINIT:
	    struct iwreq          wrq;
	CODE:
           memset(&wrq, 0, sizeof(wrq));

           strcpy(wrq.ifr_name, ifname);
 
           if(ioctl(sock, SIOCGIWRATE, &wrq) >= 0) {
	      RETVAL = wrq.u.bitrate.value;
	   } else {
	      XSRETURN_UNDEF;
	   }
	OUTPUT:
	  RETVAL

double
wireless_get_frequency(sock,ifname)
	int sock;
	char *ifname;
	PREINIT:
	  struct iwreq          wrq;
	CODE:
           memset(&wrq, 0, sizeof(wrq));

	   strcpy(wrq.ifr_name, ifname);
	   if(ioctl(sock, SIOCGIWFREQ, &wrq) >= 0) {
	     struct iw_freq *p = &(wrq.u.freq);
	     RETVAL = ((double)p->m) * pow(10,p->e);
           } else {
	     XSRETURN_UNDEF;
           }
	OUTPUT:
	  RETVAL

int
wireless_get_link_quality()
  PREINIT:
    FILE *fp = fopen("/proc/net/wireless", "r");
    char buf[64], *p = buf + 15;
  CODE:
    if(fp == NULL) {
	XSRETURN_UNDEF;
    }

    fgets(buf, sizeof(buf), fp);
    fgets(buf, sizeof(buf), fp);
    fgets(buf, sizeof(buf), fp);

    *(p + 2) = '\0';

    RETVAL = atoi(p);
  OUTPUT:
 	RETVAL

